function [n0,n1,n2,nltype1,nltype2,x,d,dclean] = generatedata(datatype,MC)

% Written by Deniz Erdogmus on 10.11.2001
% Use at your own risk
% This function generates training data for MLPs datatype specifies the type of problem data is to be drawn from

% datatype : select the problem for the training data
% MC       : tell how many random initial weights you want for your MonteCarlo runs
% when this program is run, it will generate a *.mat file containing the training data pairs {x,d} 
% and randomly chosen initial weights (MC of them)
% It will also contain the structural parameters of the MLP (n0:#inputs,n1:#neurons,n2:#outputs,nltypei:nonlinearity_type_for_each_layer)

if nargin < 2,MC = 1;end,

if datatype == 1	% sum of sinusoids
   % set the size of the MLP
   n0 = 3;n1 = 5;n2 = 1;nltype1 = 1;nltype2 = 0;   
   t = [0:0.01:1];signal = 4*sin(2*pi*t)-1.5*cos(2*pi*3*t)+0.3*sin(2*pi*5*t);
	% form input-output pairs in the prediction framework
	N = length(t)-n0-n1; % # of samples
	for s = 1:N-n0-n2+1
   	x(:,s) = signal(1,s:s+n0-1)'; % must be a column vector
	   for j = 1:n2,d(j,s) = signal(1,s+n0-1+j);end,		% single-step, two-step,... prediction
	end
	% randomly initialize weights
	W10 = 0.001*randn(n1,n0);b10 = 0.001*randn(n1,1);W20 = 0.001*randn(n2,n1);b20 = 0.001*randn(n2,1);
elseif datatype == 2	% spiral classification
	% set the size of the MLP
	n0 = 2;n1 = 40;n2 = 1;nltype1 = 1;nltype2 = 0;N = 100;
   % two spirals with different initial radii
   t = linspace(0,1*2*pi,N);	% make a couple of loops
	r0 = [2;pi];tau = 1e-2;	% initial radii and decay rate of radii
	r = r0*exp(-tau*[1:length(t)]);
	c1 = [r(1,:).*cos(t);r(1,:).*sin(t)];	% spiral 1
	c2 = [r(2,:).*cos(t);r(2,:).*sin(t)];  % spiral 2
   x = [c1,c2];	% input samples
   d = [ones(1,length(t)),-ones(1,length(t))];d = d + 0.2*randn(size(d));
	% randomly initialize weights
	W10 = 0.1*randn(n1,n0);b10 = 0.1*randn(n1,1);W20 = 0.1*randn(n2,n1);b20 = 0.1*randn(n2,1);
elseif datatype == 3	% Mackey-Glass prediction
   % set the size of the MLP
	n0 = 7;n1 = 5;n2 = 1;nltype1 = 1;nltype2 = 0;N = 200; % # of samples
   load mackey_glass_series.mat	% load training trajectory
   % form input-output pairs in the prediction framework
	for s = 1:N
   	x(:,s) = X(s:s+n0-1); % must be a column vector
	   for j = 1:n2,d(j,s) = X(s+n0-1+j);end,		% single-step, two-step,... prediction
	end
   
   % create test data
  	for s = 1:length(X)-n0+1-n2;
   	x_test(:,s) = X(s:s+n0-1); % must be a column vector
	   for j = 1:n2,d_test(j,s) = X(s+n0-1+j);end,		% single-step, two-step,... prediction
   end
   save mg_test.mat x_test d_test;
   
   % randomly initialize weights
	W10 = 0.001*randn(n1,n0);b10 = 0.001*randn(n1,1);W20 = 0.001*randn(n2,n1);b20 = 0.001*randn(n2,1);
elseif datatype == 4	% Nonlinear system identification
   % set the size of the MLP
   n0u=10;n0y=10;n0 = n0u+n0y;n1 = 10;n2 = 1;nltype1 = 1;nltype2 = 0;N = 100; % # of samples
   [d,x] = gendata(N,n0u,n0y);
	% randomly initialize weights
	W10 = 0.001*randn(n1,n0);b10 = 0.001*randn(n1,1);W20 = 0.001*randn(n2,n1);b20 = 0.001*randn(n2,1);
elseif datatype == 5	% Generalized XOR problem - parity bit
   % set the size of the MLP
   n0 = 8;n1 = 15;n2 = 1;nltype1 = 1;nltype2 = 0;N = 2^n0; % # of samples
	[dclean,x] = parsamp(N,n0);d = dclean + 0.1*randn(size(dclean));
	% randomly initialize weights
	W10 = 0.1*randn(n1,n0);b10 = 0.1*randn(n1,1);W20 = 0.1*randn(n2,n1);b20 = 0.1*randn(n2,1);
elseif datatype == 6 % laser time-series
   % set the size of the MLP
   n0 = 3;n1 = 11;n2 = 1;nltype1 = 2;nltype2 = 0;N = 1000;
   load laser_series.mat    % Laser_data
   X = Laser_data(1:N+n0+n2);
   % form input-output pairs in the prediction framework
   for s = 1:N
      x(:,s) = X(s:s+n0-1); % must be a column vector
	  for j = 1:n2,d(j,s) = X(s+n0-1+j);end,		% single-step, two-step,... prediction
   end
   for mc = 1:MC
       % randomly initialize weights
	   W1a(:,:,mc) = 0.1*randn(n1,n0);
       b1a(:,mc) = 0.1*randn(n1,1);
       W2a(:,:,mc) = 0.1*randn(n2,n1);
       b2a(:,mc) = 0.1*randn(n2,1);
       % also bring weights into vector form for MATLAB NN Toolbox
       W1t=W1a(:,:,mc);b1t=b1a(:,mc);W2t=W2a(:,:,mc);b2t=b2a(:,mc); 
       Wa(:,mc) = [W1t(:);b1t;W2t(:);b2t]; % allweights in column vector form
   end
   save laserdata.mat x d Wa W1a b1a W2a b2a n0 n1 n2 N nltype1 nltype2   
elseif datatype == 7	% Engine model identification
   % set the size of the MLP
   n0u=2;n0y=2;n0 = n0u+n0y;n1 = 5;n2 = 1;nltype1 = 2;nltype2 = 0;N = 1000; % # of samples
   [d,x] = engine(N,n0u,n0y);
   for mc = 1:MC
       % randomly initialize weights
	   W1a(:,:,mc) = 0.1*randn(n1,n0);
       b1a(:,mc) = 0.1*randn(n1,1);
       W2a(:,:,mc) = 0.1*randn(n2,n1);
       b2a(:,mc) = 0.1*randn(n2,1);
       % also bring weights into vector form for MATLAB NN Toolbox
       W1t=W1a(:,:,mc);b1t=b1a(:,mc);W2t=W2a(:,:,mc);b2t=b2a(:,mc); 
       Wa(:,mc) = [W1t(:);b1t;W2t(:);b2t]; % allweights in column vector form
   end
   save enginedata.mat x d Wa W1a b1a W2a b2a n0 n1 n2 N nltype1 nltype2   
elseif datatype == 8 % Dow Jones closing indices from 1990 to 1993
   % set the size of the MLP
   n0 = 5;n1 = 7;n2 = 1;nltype1 = 2;nltype2 = 0;N = 1000;
   load dowjones_series.mat    % dj_series
%   X = dj_series(1:N+n0+n2);
   X = dj_series(end-(N+n0+n2)+1:end);
   % form input-output pairs in the prediction framework
   for s = 1:N
      x(:,s) = X(s:s+n0-1); % must be a column vector
	  for j = 1:n2,d(j,s) = X(s+n0-1+j);end,		% single-step, two-step,... prediction
   end
   for mc = 1:MC
       % randomly initialize weights
	   W1a(:,:,mc) = 0.1*randn(n1,n0);
       b1a(:,mc) = 0.1*randn(n1,1);
       W2a(:,:,mc) = 0.1*randn(n2,n1);
       b2a(:,mc) = 0.1*randn(n2,1);
       % also bring weights into vector form for MATLAB NN Toolbox
       W1t=W1a(:,:,mc);b1t=b1a(:,mc);W2t=W2a(:,:,mc);b2t=b2a(:,mc); 
       Wa(:,mc) = [W1t(:);b1t;W2t(:);b2t]; % allweights in column vector form
   end
   save dowjonesdata.mat x d Wa W1a b1a W2a b2a n0 n1 n2 N nltype1 nltype2  
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GENERATE DATA FOR DATATYPE = 4
%%%%%%%%
function [D,X] = gendata(N,n0u,n0y)
% set initial conditions to zero
x(:,1) = [1;1];
for k = 1:N+n0u-1
   u(k) = sin(2*pi*k/10)+sin(2*pi*k/25);
   x(:,k+1) = stateeq(x(:,k),u(k));
   y(k) = outmap(x(:,k));
   if k>=n0u & k>n0y   
		X(:,k-n0u+1) = [u(k:-1:k-n0u+1)';y(k-1:-1:k-n0y)'];
      D(k-n0u+1) = y(k);
   end
end
%%%%%%%%
function xn = stateeq(xp,up)
xn(1,1) = (xp(1)/(1+xp(1)^2)+1)*sin(xp(2));
xn(2,1) = xp(2)*cos(xp(2))+xp(1)*exp(-(xp(1)^2+xp(2)^2)/8)+up^3/(1+up^2+0.5*cos(xp(1)+xp(2)));
%%%%%%%%
function yn = outmap(xp)
yn = xp(1)/(1+0.5*sin(xp(2)))+xp(2)/(1+0.5*sin(xp(1)));
%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GENERATE DATA FOR DATATYPE = 5
%%%%%%%%
function [d,x] = parsamp(N,n0)
% generates the parity bit of an n0-length sequence
% d = parity bits
% x = bit sequences
% N number of samples
% n0 = bit sequence length
if N<2^n0
    bitseq = 0.5*(sign(rand(n0,N)-0.5)+1);
    d = mod(sum(bitseq,1),2);
    x = 2*(bitseq-0.5);
    d = 2*(d-0.5);
    [d,i] = sort(d);
    x = x(:,i);
elseif N>=2^n0
    bitseq = allseq(n0)
    d = mod(sum(bitseq,1),2);
    x = 2*(bitseq-0.5);
    d = 2*(d-0.5);
    [d,i] = sort(d);
    x = x(:,i);
end    
%%%%%%%% 
function out = allseq(n0);
% generate all possible sequences of length n0
N = 2^n0;
out = [];
for r = 1:n0
    out(r,:) = zeros(1,N);
    t1 = ones(1,N/2^r);
    t2 = zeros(1,N/2^r);
    for rc = 1:2^r,
        if mod(rc,2)==1,
            out(r,(rc-1)*N/2^r+1:rc*N/2^r) = t1;
        elseif mod(rc,2)==0,
            out(r,(rc-1)*N/2^r+1:rc*N/2^r) = t2;
        end
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GENERATE DATA FOR DATATYPE = 7
function [D,X] = engine(N,n0u,n0y)
measurementnoise = 0;
m = 1;	% # of inputs
n = 2;	% # of states
l = 1;	% # of outputs
x(:,1)=[0.3e5;298]+0*[0.1e5*randn;10*randn]; % initialize state vector
Uu4=2*sin(linspace(0,4*pi,N))+10+1*randn(1,N); % generate input signal
for k = 1:N
   u(1,k) = Uu4(1,k);	% input
   [x(:,k+1),y(:,k)] = fh(x(:,k),u(:,k),k);	% iterate system
   y(:,k) = y(:,k) + measurementnoise*3e3*randn(m,1);		% add noise to measurements
   if k>=n0u & k>n0y   
	  X(:,k-n0u+1) = [u(k:-1:k-n0u+1)';y(k-1:-1:k-n0y)'];
      D(k-n0u+1) = y(1,k);
   end
end   
%%%%%%%%%%%%
function [f,h] = fh(xk,uk,k)
% system dynamics - given the instantaneous state and input and time evaluate x(k+1) and y(k)
% Two state engine manifold model
	  hr=1.4;     %specific heat ratio
	  Pamb=100e3; %ambient pressure (Pascal,=1 atm)
	  Rg=287;     %gas constant(J/kg/K)
	  Tamb=298;   %Ambient temperature (Kelvin)
	  hhtm=25;    %heat transfer coefficient
	  Vm =5.0000e-03; % manifold volume m^3
	  Tw=Tamb;    %assume that manifold wall temp. equal to ambient temperature
	  %%%%%%%%%%%%%%MAT FUNCTION (kg/deg) %%%%%%%%%%%%%%%%%%%%%%%%%
	  Pu=Pamb;
	  Tu=Tamb;
	  Pd=xk(1);
	  Td=xk(2);
	  %define minimum flow area
	  D=0.06;     %Throttle bore diameter
	  d=0.01;     %Throttle shaft diameter
	  a=d/D;
	  U0=0;      %throttle angle when closed
	  uk=uk/180*pi;%convert degrees to radians
	  Af=(pi*D^2)/4*((1-cos(uk)/cos(U0))+2/pi*(a/cos(uk)*(cos(uk)^2-a^2*cos(U0)^2)^0.5-cos(uk)/cos(U0)*asin(a*cos(U0)/cos(uk))-a*(1-a^2)^0.5+asin(a)));
	  sonic_det=(2/(hr+1))^(hr/(hr-1));
      if Pd/Pu>sonic_det,
			matt=Af*Pu/(Rg*Tu)^0.5*sqrt((2*hr)/(hr-1)*((Pd/Pu)^(2/hr)-(Pd/Pu)^((hr+1)/hr)));
	  else
			matt=Af*Pu/(Rg*Tu)^0.5*hr^(0.5)*(2/(hr+1))^((hr+1)/(2*(hr-1)));	  
	  end;
	  rpm=1500;
	  mat=matt/(6*rpm);  %now it is in kg/deg
	  %%%%%%%%%%%%%MAC FUNCTION (kg/deg) %%%%%%%%%%%%%%%%%%%%%%%%%%%
	  max_Pm=8.9998e+04;   
	  min_Pm=3.3252e+04;
	  max_mac=1.8557e-06;
	  min_mac=4.9090e-07;
	  %normalize Pm
	  x1n=xk(1)/(max_Pm-min_Pm)+(-min_Pm/(max_Pm-min_Pm));
	  macn=1.557e-1+1.0107*x1n+5.475e-2*x1n^2;
	  %denormalise macn
	  mac=(max_mac-min_mac)*macn+min_mac;	
	  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	  Ts=180;  %180 degrees
	  f(1)=xk(1)+Ts/Vm*((hr-1)*hhtm*(Tw-xk(2))/(6*rpm)+hr*Rg*(Tamb*mat-xk(2)*mac));
	  f(2)=xk(2)+Ts*Rg/Vm*xk(2)/xk(1)*((hr-1)/Rg*hhtm*(Tw-xk(2))/(6*rpm)+(hr*Tamb-xk(2))*mat+(1-hr)*xk(2)*mac);
	  f=f';
	  h=[1 0]*xk;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
